


#include "mapitems.h"









struct StructMapExtraTexture{
  char Type;              /* Which graphics to display */
  int  X;                  /* Position X */
  int  Y;                  /* Position Y */
  char AnimationLength;   /* Length of Animation */
  char AnimationPosition; /* Current Animation Frame */
  char AnimationHeight;   /* Height of the Animation Frames */
  char AnimationWidth;    /* Width of the Animation Frames */
};



/* Create the main mapsettings. */
class cls_Map{
private:
  unsigned char PreviewMapIndex;    // Index for the Preview Map Function
  unsigned int  MapCount;           // Number of maps available
  unsigned int  SelectedMap;        // Number of the selected map
  bool Loaded;
  struct StructMapSettings{
    char       Name[64];     // Name of the map
    char       Author[32];   // Name of the author/creator
    char       File_Tex_Gfx[128];
    char       File_Tex_LM[128];
    char       File_Tex_Ovr[128];
    char       File_Tex_Col[128];
    TGAImg     ColmapTmp;
    unsigned char *Colmap;
    GLuint     Tex_Gfx; // Graphical representation texture
    GLuint     Tex_LM;  // Lightmap
    GLuint     Tex_Ovr; // Overlay graphics
//    GLuint     Tex_Col; // Collision map
    StructSurf Rect;         // Size of the Map
    int        Width;
    int        Height;
    int        Music;        // Musical tune
  }Data;
  struct StructMapObjects{
    GLuint Tex; // Map objects (teleport, jumper)
    char   TexFile[128];
    GLuint TexTerminal;
    char   TexTerminalFile[128];
    cls_Spawn *Spawn;
    int NumSpawns;
    cls_Teleport *Teleport;
    int NumTeleports;
    cls_Terminal *Terminal;
    int NumTerminals;
  }Object;
  struct StructPickups{
    cls_PickupItem *Item;
    int NumItems;
    GLuint Tex;
    char TexFile[128];
  }PickupObject;
public:
  cls_Map();
  int LoadMap(const char *Filename);
  GLuint *GetGfx();
  GLuint *GetLM();
  unsigned char GetCol(unsigned int x, unsigned int y);
  char *GetGfxFile();
  char *GetColFile();
  char *GetLMFile();
  StructSurf GetRect();
  bool GetLoaded();
  void Unload();
  void DrawMapObjects(int pX, int pY);
  int GetSpawnX(int Num);
  int GetSpawnY(int Num);
  int GetNumTeleports();
  int GetTeleportX(int Num);
  int GetTeleportY(int Num);
  int GetTeleportToID(int Num);
  bool GetTeleportLeftsideSpawn(int Num);
  int GetNumTerminals();
  int GetTerminalX(int Num);
  int GetTerminalY(int Num);
  int GetTerminalW(int Num);
  int GetTerminalH(int Num);
  int GetTerminalNeedKey(int Num);
  int GetTerminalToID(int Num);
  const char *GetTerminalLine(int Num, int Line);
  int GetNumItems();
  int GetItemType(int Num);
  int GetItemX(int Num);
  int GetItemY(int Num);
  int PickupItem(int Num);
  int GetPickupItemValue(int Num);
  bool GetPickupItemSpawned(int Num);
  void DrawMap(int x, int y);
};
unsigned char cls_Map::GetCol(unsigned int x, unsigned int y){
  int bpp = Data.ColmapTmp.GetBPP()/8;
  //address = baseAddress + elementSize * (r*rowSize + c);
  return *((Data.Colmap + (bpp) * (x + (Data.Rect.Bottom-y)*Data.Rect.Right))+3);// * Data.Rect.Bottom) ;
}
cls_Map::cls_Map(){
  Loaded = 0;
  strcpy(Object.TexFile, "gfx/models/mapobjects.tga");
  strcpy(Object.TexTerminalFile, "gfx/models/terminal.tga");
  strcpy(PickupObject.TexFile, "gfx/models/objects.tga");
}
int cls_Map::GetNumItems(){
  return PickupObject.NumItems;
}
int cls_Map::GetItemType(int Num){
  if( PickupObject.NumItems > 0 && Num < PickupObject.NumItems )
    return PickupObject.Item[Num].GetType();
  return 0;
}
int cls_Map::GetItemX(int Num){
  if( PickupObject.NumItems > 0 && Num < PickupObject.NumItems )
    return PickupObject.Item[Num].GetX();
  return 0;
}
int cls_Map::GetItemY(int Num){
  if( PickupObject.NumItems > 0 && Num < PickupObject.NumItems )
    return PickupObject.Item[Num].GetY();
  return 0;
}
int cls_Map::PickupItem(int Num){
  if( PickupObject.NumItems > 0 && Num < PickupObject.NumItems ){
    PickupObject.Item[Num].Pickup();
    return PickupObject.Item[Num].GetType();
  }
  return -1;
}
bool cls_Map::GetPickupItemSpawned(int Num){
    if( PickupObject.NumItems > 0 && Num < PickupObject.NumItems ){
        return PickupObject.Item[Num].IsSpawned();
    }
    return 0;
}
int cls_Map::GetPickupItemValue(int Num){
  if( PickupObject.NumItems > 0 && Num < PickupObject.NumItems ){
    return PickupObject.Item[Num].GetFlagInt();
  }
  return 0;
}
int cls_Map::GetNumTeleports(){
  return Object.NumTeleports;
}
int cls_Map::GetTeleportX(int Num){
  if( Object.NumTeleports > 0 && Num < Object.NumTeleports )
    return Object.Teleport[Num].GetX();
  return 0;
}
int cls_Map::GetTeleportY(int Num){
  if( Object.NumTeleports > 0 && Num < Object.NumTeleports )
    return Object.Teleport[Num].GetY();
  return 0;
}
int cls_Map::GetTeleportToID(int Num){
  if( Object.NumTeleports > 0 && Num < Object.NumTeleports )
    return Object.Teleport[Num].GetToID();
  return 0;
}
bool cls_Map::GetTeleportLeftsideSpawn(int Num){
  if( Object.NumTeleports > 0 && Num < Object.NumTeleports )
    return Object.Teleport[Num].GetSpawnLeft();
  return 0;
}
int cls_Map::GetSpawnX(int Num){
  if( Object.NumSpawns > 0 && Num < Object.NumSpawns )
    return Object.Spawn[Num].GetX();
  return 0;
}
int cls_Map::GetSpawnY(int Num){
  if( Object.NumSpawns > 0 && Num < Object.NumSpawns )
    return Object.Spawn[Num].GetY();
  return 0;
}
void cls_Map::Unload(){
  glDeleteTextures(1, GetGfx());
  glDeleteTextures(1, &Object.Tex);
  glDeleteTextures(1, &Object.TexTerminal);
  glDeleteTextures(1, &PickupObject.Tex);
  if( Object.NumSpawns > 0 )
    free(Object.Spawn);
  Object.Spawn = NULL;
  if( Object.NumTeleports > 0 )
    free(Object.Teleport);
  Object.Teleport = NULL;
  if( PickupObject.NumItems > 0 )
    free(PickupObject.Item);
  PickupObject.Item = NULL;
  if( Object.NumTerminals > 0 )
    free(Object.Terminal);
  Object.Terminal = 0;
  Loaded = 0;
}
GLuint *cls_Map::GetGfx(){
  return &Data.Tex_Gfx;
}
GLuint *cls_Map::GetLM(){
  return &Data.Tex_LM;
}
char *cls_Map::GetGfxFile(){
  return Data.File_Tex_Gfx;
}
char *cls_Map::GetColFile(){
  return Data.File_Tex_Col;
}
char *cls_Map::GetLMFile(){
  return Data.File_Tex_LM;
}
StructSurf cls_Map::GetRect(){
  return Data.Rect;
}
bool cls_Map::GetLoaded(){
  return Loaded;
}
int cls_Map::LoadMap(const char *Filename){

  // Switch menu and show loading progress
//  Menu.SetMenuSelection(MENU_LOADING_MAP);

  // Create the correct filename extensions
  strcpy(Data.File_Tex_Gfx,Filename);
  strcat(Data.File_Tex_Gfx,"gfx.tga");
  strcpy(Data.File_Tex_Col,Filename);
  strcat(Data.File_Tex_Col,"col.tga");

  // File pointer for reading the mapfile
  FILE *fp;

  // Open mapfile
  if( (fp = fopen(Filename, "r")) == NULL ){
    printf("File Not Found : %s\n",Filename);
    return 1;
  }

  // Temporary variables used for loading data
  char dp[32]; // For text strings
  char tt1[64], tt2[64];
  int x, y; // X and Y position of objects
  int flag1, flag2, flag3, flag4, flag5; // Object ID and Target ID used for Teleports, Spawns etc

  // Get name of the map
  if( fgets(dp,32,fp) == NULL ) return 1;
  memset(Data.Name,'\0',64);
  strncpy(Data.Name,dp,strlen(dp)-1);

  // Who created the map?
  if( fgets(dp,32,fp) == NULL ) return 1;
  memset(Data.Author,'\0',32);
  strncpy(Data.Author,dp,strlen(dp)-1);

  // Get map width and height
  if( fscanf(fp,"%i",&Data.Width) <= 0 ) return 1;
  if( fscanf(fp,"%i",&Data.Height) <= 0 ) return 1;

  // Set up the surface rectangle
  Data.Rect.Left = 0;
  Data.Rect.Right = Data.Width*32;
  Data.Rect.Top = 0;
  Data.Rect.Bottom = Data.Height*32;

  // Get musical score used for the map
  if( fscanf(fp,"%i",&Data.Music) <= 0 ) return 1;

  // Start music
  Sound.PlayTune(Data.Music);

  // Load map graphics
  GFX.LoadTexture(GetGfxFile(),GetGfx());
  GFX.LoadTexture(Object.TexFile, &Object.Tex);
  GFX.LoadTexture(Object.TexTerminalFile, &Object.TexTerminal);
  GFX.LoadTexture(PickupObject.TexFile, &PickupObject.Tex);

  // Load collision map to memory
  Data.ColmapTmp.Load(GetColFile());
  Data.Colmap = Data.ColmapTmp.GetImg();

  // Reset all objects
  Object.NumSpawns = 0;
  Object.Spawn = NULL;
  Object.NumTeleports = 0;
  Object.Teleport = NULL;
  Object.NumTerminals = 0;
  Object.Terminal = NULL;
  PickupObject.NumItems = 0;
  PickupObject.Item = NULL;

  // Print out some map information
  printf("%s\n",Data.Name);
  printf("%s\n",Data.Author);
  printf("Size %i x %i = %i x %i\n",Data.Rect.Right,Data.Rect.Bottom,Data.Width,Data.Height);

  int cx, cy;
  float tx, ty;
  cx = Data.Rect.Right/32;
  cy = Data.Rect.Bottom/32;
  tx = 1.0/cx;
  ty = 1.0/cy;
  printf("cx: %i\ncy: %i\ntx: %f\nty: %f\n",cx,cy,tx,ty);

  // Start loading map objects
  printf("Loading Objects...\n");
  while( (fscanf(fp,"%s",dp)) > 0 ){
    if( strcmp(dp, "SPAWN") == 0 ){ // Found Spawn
      printf("Loading SPAWN Object ");
      if(fscanf(fp,"%i %i %i %i", &x, &y, &flag1, &flag2) <= 0)
        return 1;
      printf("%i located at (%i, %i)\n",Object.NumSpawns,x,y);
      Object.Spawn = (cls_Spawn*) realloc(Object.Spawn, sizeof(cls_Spawn)*(Object.NumSpawns+1));
      if( Object.Spawn == NULL )
        return 1;
      Object.Spawn[Object.NumSpawns].SetSpawn(x,y,0,flag1,flag2);
      Object.NumSpawns++;
    } else if( strcmp(dp, "TELEPORT") == 0 ){ // Found Teleport
      printf("Loading TELEPORT Object ");
      if(fscanf(fp,"%i %i %i %i %i", &x, &y, &flag1, &flag2, &flag3) <= 0)
        return 1;
      printf("%i located at (%i, %i) goes to ID %i. Need key: %i\n",Object.NumTeleports,x,y,flag1,flag2);
      Object.Teleport = (cls_Teleport*) realloc(Object.Teleport, sizeof(cls_Teleport)*(Object.NumTeleports+1));
      if( Object.Teleport == NULL )
        return 1;
      Object.Teleport[Object.NumTeleports].SetTeleport(x,y,0,flag1,flag2,12,flag3); // 12 is the animation length
      Object.NumTeleports++;
    } else if( strcmp(dp, "DISC") == 0 ){ // Found Data Disc
      printf("Loading DISC Object, located at ");
      if(fscanf(fp,"%i %i %i %i", &x, &y, &flag1, &flag2) <= 0)
        return 1;
      printf("(%i, %i)\n",x,y);
      PickupObject.Item = (cls_PickupItem*) realloc(PickupObject.Item, sizeof(cls_PickupItem)*(PickupObject.NumItems+1));
      if( PickupObject.Item == NULL )
        return 1;
      PickupObject.Item[PickupObject.NumItems].SetItem(x, y, ITEM_DISC, "Got Data Disc", flag1, 14, flag2);
      PickupObject.NumItems++;
    } else if( strcmp(dp, "BLASTER") == 0 ){ // Found Blaster Weapon
      printf("Loading BLASTER Object, located at ");
      if(fscanf(fp,"%i %i %i %i", &x, &y, &flag1, &flag2) <= 0)
        return 1;
      printf("(%i, %i)\n",x,y);
      PickupObject.Item = (cls_PickupItem*) realloc(PickupObject.Item, sizeof(cls_PickupItem)*(PickupObject.NumItems+1));
      if( PickupObject.Item == NULL )
        return 1;
      PickupObject.Item[PickupObject.NumItems].SetItem(x, y, ITEM_BLASTER, "Got Blaster Weapon", flag1, 13, flag2);
      PickupObject.NumItems++;
    } else if( strcmp(dp, "CAP") == 0 ){ // Found Capacitor
      printf("Loading CAPACITOR Object, located at ");
      if(fscanf(fp,"%i %i %i %i", &x, &y, &flag1, &flag2) <= 0)
        return 1;
      printf("(%i, %i)\n",x,y);
      PickupObject.Item = (cls_PickupItem*) realloc(PickupObject.Item, sizeof(cls_PickupItem)*(PickupObject.NumItems+1));
      if( PickupObject.Item == NULL )
        return 1;
      PickupObject.Item[PickupObject.NumItems].SetItem(x, y, ITEM_CAPACITOR, "Got Capacitor Upgrade", flag1, 11, flag2);
      PickupObject.NumItems++;
    } else if( strcmp(dp, "BATTERY") == 0 ){ // Found Battery
      printf("Loading BATTERY Object, located at ");
      if(fscanf(fp,"%i %i %i %i", &x, &y, &flag1, &flag2) <= 0)
        return 1;
      printf("(%i, %i)\n",x,y);
      PickupObject.Item = (cls_PickupItem*) realloc(PickupObject.Item, sizeof(cls_PickupItem)*(PickupObject.NumItems+1));
      if( PickupObject.Item == NULL )
        return 1;
      PickupObject.Item[PickupObject.NumItems].SetItem(x, y, ITEM_BATTERY, "Got Some Battery Power", flag1, 5, flag2);
      PickupObject.NumItems++;
    } else if( strcmp(dp, "ARMOR") == 0 ){ // Found Blaster Weapon
      printf("Loading ARMOR Object, located at ");
      if(fscanf(fp,"%i %i %i %i", &x, &y, &flag1, &flag2) <= 0)
        return 1;
      printf("(%i, %i)\n",x,y);
      PickupObject.Item = (cls_PickupItem*) realloc(PickupObject.Item, sizeof(cls_PickupItem)*(PickupObject.NumItems+1));
      if( PickupObject.Item == NULL )
        return 1;
      PickupObject.Item[PickupObject.NumItems].SetItem(x, y, ITEM_ARMOR, "Got Some Armor Upgrade", flag1, 8, flag2);
      PickupObject.NumItems++;
    } else if( strcmp(dp, "TERMINAL") == 0 ){ // Found a Terminal
      printf("Loading TERMINAL Object located at ");
      if(fscanf(fp,"%i %i %i %i %i %i %i", &x, &y, &flag1, &flag2, &flag3, &flag4, &flag5) <= 0)
        return 1;
      printf("(%i, %i)\n",x,y);
      Object.Terminal = (cls_Terminal*) realloc(Object.Terminal, sizeof(cls_Terminal)*(Object.NumTerminals+1));
      if( Object.Terminal == NULL )
        return 1;
      Object.Terminal[Object.NumTerminals].SetTerminal(x, y, 0, flag1, flag2, 0, flag3, flag4, flag5);
      for( x = 0; x < 5; x++ ){
        if( fgets(tt1,64,fp) == NULL ) return 1;
        memset(tt2,'\0',64);
        strncpy(tt2,tt1,strlen(tt1)-1);
        if( x > 0)
          Object.Terminal[Object.NumTerminals].SetInitText(x-1,tt2);
        printf("%s",tt2);
      }
      Object.NumTerminals++;
    } else {
    }
  }

  fclose(fp);

  Loaded = 1;

  GUI.OSDMessage(Data.Name,4);
  
  return 0;
}
void cls_Map::DrawMapObjects(int pX, int pY){

  int i,x,y,w,h,s,t;

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glOrtho(0,AC.GetScreenWidth(),AC.GetScreenHeight(),0,-1,1);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  for( i = 0; i < Object.NumTeleports; i++ ){
    x = Object.Teleport[i].GetX();
    y = Object.Teleport[i].GetY();
    s = Object.Teleport[i].Animation(1);
    
    glLoadIdentity();
    glTranslated(AC.GetHalfW()-pX-16+x,AC.GetHalfH()-pY-16+y,0);
    glBindTexture(GL_TEXTURE_2D,Object.Tex);
    glColor4f(1.0f,1.0f,1.0f,1.0f);
    glBegin(GL_QUADS);
      glTexCoord2f(0.0625*s,        0.8125-0.0625); glVertex3f(  0, 32, 0);
      glTexCoord2f(0.0625*s+0.0625, 0.8125-0.0625); glVertex3f( 32, 32, 0);
      glTexCoord2f(0.0625*s+0.0625, 0.8750-0.0625); glVertex3f( 32,  0, 0);
      glTexCoord2f(0.0625*s,        0.8750-0.0625); glVertex3f(  0,  0, 0);
    glEnd();
    
  }

  for( i = 0; i < Object.NumTerminals; i++ ){
    x = Object.Terminal[i].GetX();
    y = Object.Terminal[i].GetY();
    w = Object.Terminal[i].GetW();
    h = Object.Terminal[i].GetH();
    s = Object.Terminal[i].GetType();
    glLoadIdentity();
    glTranslated(AC.GetHalfW()-pX-16+x,AC.GetHalfH()-pY-16+y,0);
    glBindTexture(GL_TEXTURE_2D,Object.TexTerminal);
    glColor4f(1.0f,1.0f,1.0f,1.0f);
    glBegin(GL_QUADS);
      glTexCoord2f(0.5*s,     0.0); glVertex3f( 0, h, 0);
      glTexCoord2f(0.5*s+0.5, 0.0); glVertex3f( w, h, 0);
      glTexCoord2f(0.5*s+0.5, 1.0); glVertex3f( w, 0, 0);
      glTexCoord2f(0.5*s,     1.0); glVertex3f( 0, 0, 0);
    glEnd();
  }

  for( i = 0; i < PickupObject.NumItems; i++ ){
    if( PickupObject.Item[i].IsSpawned() ){
      x = PickupObject.Item[i].GetX();
      y = PickupObject.Item[i].GetY();
      s = PickupObject.Item[i].Animation(1);
      t = 16-PickupObject.Item[i].GetAnimGfx();
      glLoadIdentity();
      glTranslated(AC.GetHalfW()-pX-16+x,AC.GetHalfH()-pY-16+y,0);
      glBindTexture(GL_TEXTURE_2D,PickupObject.Tex);
      glColor4f(1.0f,1.0f,1.0f,1.0f);
      glBegin(GL_QUADS);
        glTexCoord2f(0.125*s,       0.0625*t);        glVertex3f(  0, 32, 0);
        glTexCoord2f(0.125*s+0.125, 0.0625*t);        glVertex3f( 32, 32, 0);
        glTexCoord2f(0.125*s+0.125, 0.0625*t+0.0625); glVertex3f( 32,  0, 0);
        glTexCoord2f(0.125*s,       0.0625*t+0.0625); glVertex3f(  0,  0, 0);
      glEnd();
    } else {
      PickupObject.Item[i].CountSpawnTime();
    }
  }

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

}

int cls_Map::GetNumTerminals(){
  return Object.NumTerminals;
}
int cls_Map::GetTerminalX(int Num){
  if( Object.NumTerminals > 0 && Num < Object.NumTerminals )
    return Object.Terminal[Num].GetX();
  return 0;
}
int cls_Map::GetTerminalY(int Num){
  if( Object.NumTerminals > 0 && Num < Object.NumTerminals )
    return Object.Terminal[Num].GetY();
  return 0;
}
int cls_Map::GetTerminalW(int Num){
  if( Object.NumTerminals > 0 && Num < Object.NumTerminals )
    return Object.Terminal[Num].GetW();
  return 0;
}
int cls_Map::GetTerminalH(int Num){
  if( Object.NumTerminals > 0 && Num < Object.NumTerminals )
    return Object.Terminal[Num].GetH();
  return 0;
}
int cls_Map::GetTerminalNeedKey(int Num){
  if( Object.NumTerminals > 0 && Num < Object.NumTerminals )
    return Object.Terminal[Num].GetKeyNeeded();
  return 0;
}
int cls_Map::GetTerminalToID(int Num){
  if( Object.NumTerminals > 0 && Num < Object.NumTerminals )
    return Object.Terminal[Num].GetToID();
  return 0;
}
const char *cls_Map::GetTerminalLine(int Num, int Line){
  if( Object.NumTerminals > 0 && Num < Object.NumTerminals )
    return Object.Terminal[Num].GetInitText(Line);
  return "";
}
void cls_Map::DrawMap(int x, int y){

  int cx, cy;
  float tx, ty;

  int res = 32;
  
  cx = Data.Rect.Right/res;
  cy = Data.Rect.Bottom/res;
  tx = 1.0/res;
  ty = 1.0/res;

  int j,k;

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glOrtho(0,AC.GetScreenWidth(),AC.GetScreenHeight(),0,-1,1);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glTranslated(x,y,0);
  glBindTexture(GL_TEXTURE_2D, Data.Tex_Gfx);
  glColor4f(1.0f,1.0f,1.0f,1.0f);
  for( k = 0; k < cy; k++ ){
    for( j = 0; j < cx; j++ ){
      glBegin(GL_QUADS);
        glTexCoord2f(tx*j,    (1-ty)-k*ty); glVertex3f( cx*j,    cy*k+cy, 0);
        glTexCoord2f(tx*j+tx, (1-ty)-k*ty); glVertex3f( cx*j+cx, cy*k+cy, 0);
        glTexCoord2f(tx*j+tx,  1-ty*k);     glVertex3f( cx*j+cx, cy*k,    0);
        glTexCoord2f(tx*j,     1-ty*k);     glVertex3f( cx*j,    cy*k,    0);
      glEnd();
    }
  }
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

  

}



cls_Map Map;