/* mapitems.cpp
 *
 * This file contains all needed classes for mapobjects (pickups, teleporters, spawns etc)
 *
 */






#define ITEM_DISC 0
#define ITEM_BLASTER 1
#define ITEM_CAPACITOR 2
#define ITEM_BATTERY 3
#define ITEM_ARMOR 4


/* cls_Spawn
 * ============================================
 * This class holds all needed information for
 * spawns on the map.
 * ============================================ */
class cls_Spawn{
private:
  int  X;
  int  Y;
  bool Busy;
  int  Team;
  int  Time;
public:
  cls_Spawn();
  void SetSpawn(int sX, int sY, bool sBusy, int sTeam, int sTime);
  int GetX();
  int GetY();
};
cls_Spawn::cls_Spawn(){
  X    = 0;
  Y    = 0;
  Busy = 0;
  Team = 0;
  Time = 0;
}
int cls_Spawn::GetX(){
  return X;
}
int cls_Spawn::GetY(){
  return Y;
}
void cls_Spawn::SetSpawn(int sX, int sY, bool sBusy, int sTeam, int sTime){
  X = sX;
  Y = sY;
  Busy = sBusy;
  Team = sTeam;
  Time = sTime;
}



/* cls_Teleport
 * ============================================
 * This class holds all needed information for
 * teleports on the map.
 * ============================================ */
class cls_Teleport{
private:
  int  X;
  int  Y;
  bool Busy;
  int  ToID;
  bool NeedKey;
  int  AnimNum;
  float AnimStep;
  int  AnimLen;
  bool SpawnLeft;
public:
  cls_Teleport();
  void SetTeleport(int sX, int sY, bool sBusy, int sToID, bool sNeedKey, int sAnimLen, bool sSpawn);
  int  GetX();
  int  GetY();
  int  GetToID();
  bool GetKeyNeeded();
  int  Animation(bool Step);
  bool GetSpawnLeft();
};
cls_Teleport::cls_Teleport(){
  X       = 0;
  Y       = 0;
  Busy    = 0;
  ToID    = 0;
  NeedKey = 0;
  AnimLen = 12;
  AnimStep = 0;
  AnimNum = 0;
}
int cls_Teleport::Animation(bool Step){
  if( Step ){
    AnimStep+=0.3;
    if( AnimStep > 1 ){
      AnimNum++;
      if( AnimNum > AnimLen-1 )
        AnimNum = 0;
      AnimStep = 0;
    }
    return AnimNum;
  } else {
    return AnimNum;
  }
}
int cls_Teleport::GetX(){
  return X;
}
int cls_Teleport::GetY(){
  return Y;
}
int cls_Teleport::GetToID(){
  return ToID;
}
bool cls_Teleport::GetKeyNeeded(){
  return NeedKey;
}
bool cls_Teleport::GetSpawnLeft(){
  return SpawnLeft;
}
void cls_Teleport::SetTeleport(int sX, int sY, bool sBusy, int sToID, bool sNeedKey, int sAnimLen, bool sSpawn){
  X = sX;
  Y = sY;
  Busy = sBusy;
  ToID = sToID;
  NeedKey = sNeedKey;
  AnimLen = sAnimLen;
  SpawnLeft = sSpawn;
}




/* cls_PickupItem
 * ============================================
 * This class holds all needed information for
 * pickup objects.
 * ============================================ */
class cls_PickupItem{
private:
  int   X;
  int   Y;
  int   Type;
  bool  Spawned;
  float SpawnTime;
  int   SpawnTimeMax;
  bool  ReSpawn;
  int   AnimGfx;
  int   AnimNum;
  float AnimStep;
  int   AnimLen;
  char  FlagStr[32];
  int   FlagInt;
public:
  cls_PickupItem();
  void SetItem(int sX, int sY, int sType, const char *sFs, int sFi, int sAnimGfx, bool sRespawn);
  int GetX();
  int GetY();
  int GetType();
  int Pickup();
  const char *GetFlagStr();
  int GetFlagInt();
  int Animation(bool Step);
  int GetAnimGfx();
  bool IsSpawned();
  void CountSpawnTime();
};

cls_PickupItem::cls_PickupItem(){

}
void cls_PickupItem::SetItem(int sX, int sY, int sType, const char *sFs, int sFi, int sAnimGfx, bool sRespawn){
  X = sX;
  Y = sY;
  Type = sType;
  Spawned = 1;
  SpawnTime = 0;
  SpawnTimeMax = 15;
  ReSpawn = sRespawn; // 0 if object will not respawn!

  strcpy(FlagStr, sFs);
  FlagInt = sFi;

  AnimGfx = sAnimGfx;
  AnimNum = 0;
  AnimLen = 8;
  AnimStep = 0;
}
void cls_PickupItem::CountSpawnTime(){
  if( ReSpawn )
    SpawnTime+=0.01;
  if( SpawnTime > SpawnTimeMax )
    Spawned = 1;
}
int cls_PickupItem::GetAnimGfx(){
  return AnimGfx;
}
bool cls_PickupItem::IsSpawned(){
  return Spawned;
}
int cls_PickupItem::Animation(bool Step){
  if( Step ){
    AnimStep+=0.3;
    if( AnimStep > 1 ){
      AnimNum++;
      if( AnimNum > AnimLen-1 )
        AnimNum = 0;
      AnimStep = 0;
    }
    return AnimNum;
  } else {
    return AnimNum;
  }
}
int cls_PickupItem::GetX(){
  return X;
}
int cls_PickupItem::GetY(){
  return Y;
}
int cls_PickupItem::GetType(){
  return Type;
}
int cls_PickupItem::Pickup(){
  if( Spawned ){
    Spawned = 0;
    SpawnTime = 0;
    GUI.OSDMessage(FlagStr, 4);
    return Type;
  } else {
    return -1;
  }
}
int cls_PickupItem::GetFlagInt(){
  return FlagInt;
}














/* cls_Terminal
 * ============================================
 * This class holds all needed information for
 * terminals on the map
 * ============================================ */
class cls_Terminal{
private:
  int  X;
  int  Y;
  int  W;
  int  H;
  bool Busy;
  int  ToID;
  int  NeedKey;
  int  AnimNum;
  float AnimStep;
  int  AnimLen;
  int  Type;
  struct StructWelcome{
    char Line[64];
  }InitText[4];
public:
  cls_Terminal();
  void SetTerminal(int sX, int sY, bool sBusy, int sToID, int sNeedKey, int sAnimLen, int sW, int sH, int sType);
  void SetInitText(int Line, const char *Text);
  const char *GetInitText(int Line);
  int  GetX();
  int  GetY();
  int  GetW();
  int  GetH();
  int  GetToID();
  int  GetType();
  int  GetKeyNeeded();
  int  Animation(bool Step);
};
cls_Terminal::cls_Terminal(){
  X       = 0;
  Y       = 0;
  W       = 0;
  H       = 0;
  Busy    = 0;
  ToID    = 0;
  NeedKey = 0;
  AnimLen = 12;
  AnimStep = 0;
  AnimNum = 0;
  Type    = 0;
}
void cls_Terminal::SetInitText(int Line, const char *Text){
  memset(InitText[Line].Line,'\0',64);
  strcpy(InitText[Line].Line,Text);
}
const char *cls_Terminal::GetInitText(int Line){
  return InitText[Line].Line;
}
int cls_Terminal::Animation(bool Step){
  if( Step ){
    AnimStep+=0.3;
    if( AnimStep > 1 ){
      AnimNum++;
      if( AnimNum > AnimLen-1 )
        AnimNum = 0;
      AnimStep = 0;
    }
    return AnimNum;
  } else {
    return AnimNum;
  }
}
int cls_Terminal::GetX(){
  return X;
}
int cls_Terminal::GetY(){
  return Y;
}
int cls_Terminal::GetW(){
  return W;
}
int cls_Terminal::GetH(){
  return H;
}
int cls_Terminal::GetToID(){
  return ToID;
}
int cls_Terminal::GetType(){
  return Type;
}
int cls_Terminal::GetKeyNeeded(){
  return NeedKey;
}
void cls_Terminal::SetTerminal(int sX, int sY, bool sBusy, int sToID, int sNeedKey, int sAnimLen, int sW, int sH, int sType){
  X = sX;
  Y = sY;
  W = sW;
  H = sH;
  Busy = sBusy;
  ToID = sToID;
  NeedKey = sNeedKey;
  AnimLen = sAnimLen;
  Type = sType;
}