










#define T_RED                   0 /* Team = Red                          */
#define T_BLUE                  1 /* Team = Blue                         */
#define T_YELLOW                2 /* Team = Yellow                       */
#define T_GREEN                 3 /* Team = Green                        */

#define CHAR_ANIMATION_STANDING                 0 /* Standing, first row in animation board */
#define CHAR_ANIMATION_RUNNING                  1 /* Running, second row in animation board */
#define CHAR_ANIMATION_FIRING                   2 /* Standing still and firing */
#define CHAR_ANIMATION_FIRING_RUNNING           3 /* Running and firing */
#define CHAR_ANIMATION_FIRING_RRUNNING_BACK     4 /* Running backwards and firing (not used for the moment) */
#define CHAR_ANIMATION_JUMP_ASSIST              5 /* Using jump assist */
#define CHAR_ANIMATION_IN_THE_AIR               6 /* Flying in the air (or falling) */
#define CHAR_ANIMATION_FIRING_IN_THE_AIR        7 /* Firing in the air (or falling + firing) */
#define CHAR_ANIMATION_AIR_SHAFT                8 /* Using the airshaft (flaps) */
#define CHAR_ANIMATION_JUMPING                 18 /* Jumping */
#define CHAR_ANIMATION_JUMPING_AND_RUNNING     19 /* Jumping when running */
#define CHAR_ANIMATION_JUMPING_AND_RUNNING_BW  20 /* Jumping when running backwards (not used) */
#define CHAR_ANIMATION_CROUCHING               21 /* Crouching */
#define CHAR_ANIMATION_CRAWLING                22 /* Crawling */

#define CHAR_ON_THE_GROUND  0
#define CHAR_IN_THE_AIR     1
#define CHAR_CROUCHING      2
#define CHAR_FREEZE         3
#define CHAR_JETPACK        4
#define CHAR_FIRING         5
#define CHAR_DEAD          99

#define CHAR_MOVE_STOP  0
#define CHAR_MOVE_LEFT  1
#define CHAR_MOVE_RIGHT 2





#define CHAR_ITEM_NUMBER_OF_DISCS 4


class cls_Character{
private:
  GLuint Tex;
  char Filename[64];
  bool TextureLoaded;
  struct StructAnimation{
    int  AnimationState;
    float PosInc;
    int  Position;
    int  Length;
    bool ReverseFlag;
    int  Model;
    int  Menu;  /* Only for selecting character in menues */
  }Animation; /* Animation info */
  struct StructPhysics{
    struct StructPosition{
      unsigned int X;
      unsigned int Y;
    }Position;
    struct StructSpeed{
      float XSpeed;
      float YSpeed;
      float XMax;
      float XSpeedTarget;
      float YMax;
      float YMin;
    }Speed;
    int State;
  }Physics;   /* Position, speed, etc */
  struct StructAboutPlayer{
    char Name[32];
    char TeamName[32];
    int  Team;
  }About;     /* Name, Teamname, etc */
  struct StructPlayerSettings{
    int Crosshair;
    int CrosshairAnimation;
  }Settings;  /* Player Custom settings */
  struct StructPlayerStatus{
    int Health;
    int Armor;
    int ArmorMax;
    int Frags;
    int Deaths;
    int SpawnTime;
  }Status;    /* Health, frags, etc */
  struct StructPlayerItems{
    bool HeartSensor;
    bool Capacitor;
    bool GotSuit;
    bool GotDisk[CHAR_ITEM_NUMBER_OF_DISCS];
  }Items;
  struct StructWeapons{
    bool Blaster;
    bool Grenades;
    bool Railgun;
    bool Trailgun;
    int Ammo_Blaster;
    int Ammo_Grenades;
    int Ammo_BatteryPower;
    int Ammo_Blaster_Max;
    int Ammo_Grenades_Max;
    int Ammo_BatteryPower_Max;
  }Weapon;
  bool CanEnterTerminal;
  int ActiveTerminalID;
  int TerminalID;
  int TerminalScreen;
public:
  cls_Character();
  void SetName(const char *Name);
  const char *GetName();
  void SetTeam(int TeamNo, const char *TeamName);
  const char *GetTeamName();
  int GetTeamNo();
  void SetAnimation(int NewState, int Length, int Menu, int Model, int Position, bool ReverseFlag);
  void StepAnimationFrame(bool Backward);
  void SetStatus(int Health, int Armor, int ArmorMax, int Frags, int Deaths, int SpawnTime);
  void ChangeHealth(int Change);
  void SetCrosshair(int Crosshair);
  void SpawnPlayer(int X, int Y);
  void PrintDebugInfo();
  int GetPosX();
  void SetSpeedX(int RelIncX);
  int GetPosY();
  void SetSpeedY(int RelIncY);
  void PerformPhysics();
  void SetItem(int Item, int Index, bool Val);
  GLuint *GetTex();
  char *GetFilename();
  void BuildAnimations();
  void KillTexture();
  void DrawPlayer();
  void Jump();
  void Move(int MF);
  void Crouch(bool On);
  void Fire();
  void ActionEnterTerminal();
  void ActionLeaveTerminal();
  int GetActiveTerminalID();
  void SetTerminalScreen(int NewScreen);
  int GetTerminalScreen();
  int GetHealth();
  int GetArmor();
  int GetBattery();

};
GLuint *cls_Character::GetTex(){
  return &Tex;
}
void cls_Character::Crouch(bool On){
  if( On ){
    Physics.State = CHAR_CROUCHING;
    SetAnimation(CHAR_ANIMATION_CROUCHING,1,0,0,0,Animation.ReverseFlag);
  } else {
    Physics.State = CHAR_IN_THE_AIR; // To prevent crouch->jump abuse.
    SetAnimation(CHAR_ANIMATION_IN_THE_AIR,1,0,0,0,Animation.ReverseFlag);
  }
}
int cls_Character::GetHealth(){
  return Status.Health;
}
int cls_Character::GetArmor(){
  return Status.Armor;
}
int cls_Character::GetBattery(){
  return Weapon.Ammo_BatteryPower;
}
void cls_Character::ActionEnterTerminal(){
  int i;
  if( CanEnterTerminal ){
    if( Map.GetTerminalNeedKey(TerminalID) == -1 ){
      AC.SpecifyLoop(LOOP_TERMINAL);
      ActiveTerminalID = Map.GetTerminalToID(TerminalID);
      TerminalScreen = 0;
      GUI.ClearConsole();
      for( i = 1; i < 4; i++ ){
        GUI.AddConsoleLine(Map.GetTerminalLine(TerminalID,i),1);
      }
      return;
    } else {
      if( Items.GotDisk[Map.GetTerminalNeedKey(TerminalID)] ){
        AC.SpecifyLoop(LOOP_TERMINAL);
        ActiveTerminalID = Map.GetTerminalToID(TerminalID);
        TerminalScreen = 0;
        GUI.ClearConsole();
        for( i = 1; i < 4; i++ ){
          GUI.AddConsoleLine(Map.GetTerminalLine(TerminalID,i),1);
        }
        GUI.AddConsoleLine("",0);
        return;
      }
    }
    GUI.OSDMessage("Need Data Disc to Enter Terminal", 4);
  } 
}
int cls_Character::GetActiveTerminalID(){
  return TerminalID;
}
void cls_Character::ActionLeaveTerminal(){
  AC.SpecifyLoop(LOOP_GAME);
}
void cls_Character::SetTerminalScreen(int NewScreen){
  TerminalScreen = NewScreen;
}
int cls_Character::GetTerminalScreen(){
  return TerminalScreen;
}
char *cls_Character::GetFilename(){
  return Filename;
}
void cls_Character::BuildAnimations(){
  if( !TextureLoaded )
    GFX.LoadTexture(GetFilename(),GetTex());
}
void cls_Character::KillTexture(){
  if( TextureLoaded )
    glDeleteTextures(1,&Tex);
}
void cls_Character::DrawPlayer(){

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glOrtho(0,AC.GetScreenWidth(),AC.GetScreenHeight(),0,-1,1);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glTranslated(AC.GetHalfW()-16,AC.GetHalfH()-16,0);
  glBindTexture(GL_TEXTURE_2D,Tex);
  glColor4f(1.0f,1.0f,1.0f,1.0f);
  glBegin(GL_QUADS);
  if( Animation.ReverseFlag ){
    glTexCoord2f(Animation.Position*0.1+0.1, 0.9-Animation.AnimationState*0.1); glVertex3f(  0, 32, 0);
    glTexCoord2f(Animation.Position*0.1,     0.9-Animation.AnimationState*0.1); glVertex3f( 32, 32, 0);
    glTexCoord2f(Animation.Position*0.1,     1.0-Animation.AnimationState*0.1); glVertex3f( 32,  0, 0);
    glTexCoord2f(Animation.Position*0.1+0.1, 1.0-Animation.AnimationState*0.1); glVertex3f(  0,  0, 0);
  } else {
    glTexCoord2f(Animation.Position*0.1,     0.9-Animation.AnimationState*0.1); glVertex3f(  0, 32, 0);
    glTexCoord2f(Animation.Position*0.1+0.1, 0.9-Animation.AnimationState*0.1); glVertex3f( 32, 32, 0);
    glTexCoord2f(Animation.Position*0.1+0.1, 1.0-Animation.AnimationState*0.1); glVertex3f( 32,  0, 0);
    glTexCoord2f(Animation.Position*0.1,     1.0-Animation.AnimationState*0.1); glVertex3f(  0,  0, 0);
  }
  glEnd();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

}


cls_Character::cls_Character(){
  // About
  SetName("Player 1");
  SetTeam(1,"Human");
  SetAnimation(CHAR_ANIMATION_STANDING,6,0,0,0,0);
  SpawnPlayer(0,0);
  SetCrosshair(0);
  SetStatus(100,0,100,0,0,3);
  Physics.Speed.XMax = 5;
  Physics.Speed.XSpeedTarget = 0;
  Physics.Speed.YMax = 14;
  Physics.Speed.YMin = -11; // Jumping Strength
  Weapon.Ammo_BatteryPower = 0;
  Weapon.Ammo_BatteryPower_Max = 100;
  Weapon.Ammo_Blaster = 1000;
  Weapon.Ammo_Blaster_Max = 1000;
  Weapon.Ammo_Grenades = 0;
  Weapon.Ammo_Grenades_Max = 10;
  Weapon.Blaster = 0;
  Weapon.Grenades = 0;
  Weapon.Railgun = 0;
  Weapon.Trailgun = 0;
  SetItem(0,0,0);
  SetItem(1,0,0);
  SetItem(2,0,1);
  TextureLoaded = 0;
  strcpy(Filename,"gfx/models/character1.tga");
  CanEnterTerminal = 0;
  ActiveTerminalID = 0;
  int i;
  for( i = 0; i < CHAR_ITEM_NUMBER_OF_DISCS; i++ ){
    SetItem(3,i,0);
  }
  
}
void cls_Character::SetItem(int Item, int Index, bool Val){
  switch( Item ){
    case 0: // HS
      Items.HeartSensor = Val;
      break;
    case 1: // CAP
      Items.Capacitor = Val;
      break;
    case 2: // SUIT
      Items.GotSuit = Val;
      break;
    case 3:
      if( Index >= 0 && Index < CHAR_ITEM_NUMBER_OF_DISCS)
        Items.GotDisk[Index] = Val;
      break;
  }
}
void cls_Character::SetName(const char *Name)
{
  strcpy(About.Name,Name);
}
const char *cls_Character::GetName(){
  return About.Name;
}
void cls_Character::SetTeam(int TeamNo, const char *TeamName){
  strcpy(About.TeamName,TeamName);
  About.Team = TeamNo;
}
const char *cls_Character::GetTeamName(){
  return About.TeamName;
}
int cls_Character::GetTeamNo(){
  return About.Team;
}
void cls_Character::SetAnimation(int NewState, int Length, int Menu, int Model, int Position, bool ReverseFlag){
  Animation.AnimationState = NewState;
  Animation.Length = Length;
  Animation.Menu = Menu;
  Animation.Model = Model;
  Animation.Position = Position;
  Animation.ReverseFlag = ReverseFlag;
  Animation.PosInc = 0;
}
void cls_Character::StepAnimationFrame(bool Backward){
  Animation.PosInc+=0.3;

  if( Animation.PosInc > 1 ){
    if( Backward ){
      Animation.Position--;
      if( Animation.Position < 0 )
        Animation.Position = Animation.Length-1;
    } else {
      Animation.Position++;
      if( Animation.Position > Animation.Length-1 )
        Animation.Position = 0;
      if( Animation.AnimationState == CHAR_ANIMATION_FIRING )
        SetAnimation(CHAR_ANIMATION_STANDING,6,0,0,0,Animation.ReverseFlag);
      if( Animation.AnimationState == CHAR_ANIMATION_FIRING_RUNNING )
        SetAnimation(CHAR_ANIMATION_RUNNING,8,0,0,0,Animation.ReverseFlag);
    }
    Animation.PosInc = 0;
  }
  
}
void cls_Character::SetStatus(int Health, int Armor, int ArmorMax, int Frags, int Deaths, int SpawnTime){
  Status.Armor = Armor;
  Status.ArmorMax = ArmorMax;
  Status.Frags = Frags;
  Status.Deaths = Deaths;
  Status.Health = Health;
  Status.SpawnTime = SpawnTime;
}
void cls_Character::ChangeHealth(int Change){
  Status.Health += Change;
}
void cls_Character::SetCrosshair(int Crosshair){
  Settings.Crosshair = Crosshair;
  Settings.CrosshairAnimation = 0;
}
void cls_Character::SpawnPlayer(int X, int Y){
  Physics.Position.X = X;
  Physics.Position.Y = Y;
  Physics.Speed.XSpeed = 0;
  Physics.Speed.YSpeed = 0;
  Items.Capacitor = 0;
  Items.GotDisk[0] = 0;
  Items.GotDisk[1] = 0;
  Items.GotDisk[2] = 0;
  Items.GotDisk[3] = 0;
  Items.GotSuit = 0;
  Items.HeartSensor = 0;
  Weapon.Ammo_BatteryPower = 0;
  Weapon.Ammo_Blaster = 0;
  Weapon.Ammo_Grenades = 0;
  Weapon.Grenades = 0;
  Weapon.Railgun = 0;
  Weapon.Trailgun = 0;
  Physics.State = CHAR_ON_THE_GROUND;
}
void cls_Character::PrintDebugInfo(){
  printf("Character Information:\n");
  printf("Name: %s\n",GetName());
  printf("Team %i: %s\n",GetTeamNo(),GetTeamName());
  printf("Animation: %i\n",Animation.AnimationState);
}
int cls_Character::GetPosX(){
  return Physics.Position.X;
}
void cls_Character::SetSpeedX(int RelIncX){
  Physics.Speed.XSpeed = RelIncX;
}
int cls_Character::GetPosY(){
  return Physics.Position.Y;
}
void cls_Character::SetSpeedY(int RelIncY){
  Physics.Speed.YSpeed = RelIncY;
}
void cls_Character::Jump(){
  if( Physics.State == CHAR_ON_THE_GROUND ){
    Physics.Speed.YSpeed = Physics.Speed.YMin;
    Physics.State = CHAR_IN_THE_AIR;
  }
}
void cls_Character::Move(int MF){
  switch( MF ){
    case CHAR_MOVE_STOP:
      Physics.Speed.XSpeedTarget = 0;
      SetAnimation(CHAR_ANIMATION_STANDING,6,0,0,0,Animation.ReverseFlag);
      break;
    case CHAR_MOVE_LEFT:
      Physics.Speed.XSpeedTarget = -Physics.Speed.XMax;
      SetAnimation(CHAR_ANIMATION_RUNNING,8,0,0,0,1);
      break;
    case CHAR_MOVE_RIGHT:
      Physics.Speed.XSpeedTarget = Physics.Speed.XMax;
      SetAnimation(CHAR_ANIMATION_RUNNING,8,0,0,0,0);
      break;
    default:
      Physics.Speed.XSpeedTarget = 0;
      break;
  }
}
void cls_Character::Fire(){
  if( abs(Physics.Speed.XSpeed) > 2 ){
    SetAnimation(CHAR_ANIMATION_FIRING_RUNNING,7,0,0,0,Animation.ReverseFlag);
  } else {
    SetAnimation(CHAR_ANIMATION_FIRING,7,0,0,0,Animation.ReverseFlag);
  }
}
void cls_Character::PerformPhysics(){

  Physics.Speed.XSpeed = Physics.Speed.XSpeed + ((Physics.Speed.XSpeedTarget-Physics.Speed.XSpeed)/4);

  if( abs(Physics.Speed.XSpeed) < 0.2 ){
    Physics.Speed.XSpeed = 0;
  }

  int cw,ch,cw8,i;
  cw = 32;
  ch = 32;
  cw8 = cw/8;

  

  int x, y;
  x = Physics.Position.X + Physics.Speed.XSpeed;
  y = Physics.Position.Y + Physics.Speed.YSpeed;

  StructSurf MapRect = Map.GetRect();

  // Check if character is outside map, than respawn!
  if( x <= 0 || x+cw >= MapRect.Right || y <= 0 || y+ch >= MapRect.Bottom ){
    SpawnPlayer(Map.GetSpawnX(0),Map.GetSpawnY(0));
    return;
  }
  


  // ===================================================================
  // Check for left right side collision with walls
  // ===================================================================
  if( Physics.Speed.XSpeed < 0 ){
    while( Map.GetCol(x,y+ch/2) > 1 ){
      x++;
      Physics.Speed.XSpeed = 0;
    }
  } else if( Physics.Speed.XSpeed > 0 ) {
    while( Map.GetCol(x+cw,y+ch/2) > 1 ){
      x--;
      Physics.Speed.XSpeed = 0;
    }
  }
  // ===================================================================
  // Check for feet collision with walls
  // ===================================================================
  while( Map.GetCol(x+cw8,y+ch) > 1 || Map.GetCol(x+cw-cw8,y+ch) > 1 || Map.GetCol(x+cw/2,y+ch) > 1 ){
    y--;
    Physics.State = CHAR_ON_THE_GROUND;
    Physics.Speed.YSpeed = 0;
  }
  // ===================================================================
  // Check for head left collision with walls
  // ===================================================================
  while( Map.GetCol(x+cw8,y+1) > 1 || Map.GetCol(x+cw-cw8,y+1) > 1 || Map.GetCol(x+cw/2,y+1) > 1 ){
    y++;
    Physics.State = CHAR_ON_THE_GROUND;
    Physics.Speed.YSpeed = 0;
  }
  // ===================================================================
  // DONE WITH WALL CHECKS
  // ===================================================================

  // Check if the character is in the air to set correct flag,
  // and enable jump feature!
  if( Map.GetCol(x+cw/2,y+ch+16) == 0 )
    Physics.State = CHAR_IN_THE_AIR;

  // Update character position!
  Physics.Position.X = x;
  Physics.Position.Y = y;

  // ===================================================================
  // Check for collision with map objects
  // ===================================================================
  for( i = 0; i < Map.GetNumTeleports(); i++ ){
    if( x+cw/2 > Map.GetTeleportX(i) && x+cw/2 < Map.GetTeleportX(i)+cw && y+ch/2 > Map.GetTeleportY(i) && y+ch/2 < Map.GetTeleportY(i)+ch ){
      // Should Teleport!
      int toID = Map.GetTeleportToID(i);
      Physics.Position.X = Map.GetTeleportX(toID) + 32 + ((-1)*Map.GetTeleportLeftsideSpawn(toID)*64);
      Physics.Position.Y = Map.GetTeleportY(toID);
      Physics.Speed.XSpeed = 0;
      Physics.Speed.XSpeedTarget = 0;
      SetAnimation(CHAR_ANIMATION_STANDING,6,0,0,0,Animation.ReverseFlag);
    }
  }
  CanEnterTerminal = 0;
  for( i = 0; i < Map.GetNumTerminals(); i++ ){
    if( x+cw/2 > Map.GetTerminalX(i) && x+cw/2 < Map.GetTerminalX(i)+Map.GetTerminalW(i) && y+ch/2 > Map.GetTerminalY(i) && y+ch/2 < Map.GetTerminalY(i)+Map.GetTerminalH(i) ){
      // Will enter terminal?
      CanEnterTerminal = 1;
      TerminalID = i;
    }
  }

  // ===================================================================
  // Check for collision with pickup objects
  // ===================================================================
  for( i = 0; i < Map.GetNumItems(); i++ ){
    if( x+cw/2 > Map.GetItemX(i) && x+cw/2 < Map.GetItemX(i)+cw && y+ch/2 > Map.GetItemY(i) && y+ch/2 < Map.GetItemY(i)+ch ){
      // On item location!
        if( Map.GetPickupItemSpawned(i) ){
          switch( Map.PickupItem(i) ){
            case ITEM_DISC:
              Items.GotDisk[0] = 1;
              break;
            case ITEM_CAPACITOR:
              Items.Capacitor = 1;
              break;
            case ITEM_BLASTER:
              Weapon.Blaster = 1;
              Weapon.Ammo_Blaster += Map.GetPickupItemValue(i);
              if( Weapon.Ammo_Blaster > Weapon.Ammo_Blaster_Max )
                Weapon.Ammo_Blaster = Weapon.Ammo_Blaster_Max;
              break;
            case ITEM_ARMOR:
              Status.Armor += Map.GetPickupItemValue(i);
              if( Status.Armor > Status.ArmorMax )
                Status.Armor = Status.ArmorMax;
              break;
            case ITEM_BATTERY:
              Weapon.Ammo_BatteryPower += Map.GetPickupItemValue(i);
              if( Weapon.Ammo_BatteryPower > (Weapon.Ammo_BatteryPower_Max+Weapon.Ammo_BatteryPower_Max*Items.Capacitor) )
                Weapon.Ammo_BatteryPower = Weapon.Ammo_BatteryPower_Max + Weapon.Ammo_BatteryPower_Max*Items.Capacitor;
              break;
            default:
              break;
          }
      }
    }
  }

  // Update by gravity.
  Physics.Speed.YSpeed += 0.5;
  if( Physics.Speed.YSpeed > Physics.Speed.YMax )
    Physics.Speed.YSpeed = Physics.Speed.YMax;
}

cls_Character Player;